from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, eks_panther_obj_ref

amazon_eks_audit_system_namespace_from_public_ip_tests: list[RuleTest] = [
    RuleTest(
        name="non-system username",
        expected_result=False,
        log={
            "annotations": {"authorization.k8s.io/decision": "allow", "authorization.k8s.io/reason": ""},
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "35506555-dffc-4337-b2b1-c4af52b88e18",
            "kind": "Event",
            "level": "Request",
            "objectRef": {
                "apiVersion": "v1",
                "name": "kube-bench-drn4j",
                "namespace": "default",
                "resource": "pods",
                "subresource": "log",
            },
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/DevAdministrator",
                "arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000",
            ],
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["kubernetes-admin"],
            "p_event_time": "2022-11-29 00:09:04.38",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 00:10:25.067",
            "p_row_id": "2e4ab474b0f0f7a4a8fff4f014a9b32a",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 00:09:04.38",
            "requestURI": "/api/v1/namespaces/default/pods/kube-bench-drn4j/log?container=kube-bench",
            "responseStatus": {"code": 200},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseStarted",
            "stageTimestamp": "2022-11-29 00:09:04.392",
            "user": {
                "extra": {
                    "accessKeyId": ["ASIARLIVEKVNN6Y6J5UW"],
                    "arn": ["arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000"],
                    "canonicalArn": ["arn:aws:iam::123412341234:role/DevAdministrator"],
                    "sessionName": ["1669660343296132000"],
                },
                "groups": ["system:masters", "system:authenticated"],
                "uid": "aws-iam-authenticator:123412341234:AROARLIVEKVNIRVGDLJWJ",
                "username": "kubernetes-admin",
            },
            "userAgent": "kubectl/v1.25.4 (darwin/arm64) kubernetes/872a965",
            "verb": "get",
        },
    ),
    RuleTest(
        name="system username - private ip",
        expected_result=False,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["10.0.27.115"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:34:06.892",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:45:25.024",
            "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
            "responseStatus": {"code": 200},
            "sourceIPs": ["10.0.27.115"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-11-29 22:40:02.903",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
    RuleTest(
        name="403 from Public IP zero count",
        expected_result=True,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:34:06.892",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:45:25.024",
            "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
            "responseStatus": {"code": 200},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-11-29 22:40:02.903",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
    RuleTest(
        name="system username - public ip - not ResponseComplete",
        expected_result=False,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "c8c5bc49-cd5d-45d6-999c-b55783c7840f",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:46:37.995",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:50:24.942",
            "p_row_id": "fa229ed1d0b18094f4a1aff3149531",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:46:37.995",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2529923&timeout=6m41s&timeoutSeconds=401&watch=true",
            "responseStatus": {"code": 200},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseStarted",
            "stageTimestamp": "2022-11-29 22:46:38.013",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
    RuleTest(
        name="system username - public ip - 403",
        expected_result=False,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "c8c5bc49-cd5d-45d6-999c-b55783c7840f",
            "kind": "Event",
            "level": "Metadata",
            "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
            "p_any_ip_addresses": ["5.5.5.5"],
            "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
            "p_event_time": "2022-11-29 22:46:37.995",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-11-29 22:50:24.942",
            "p_row_id": "fa229ed1d0b18094f4a1aff3149531",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-11-29 22:46:37.995",
            "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2529923&timeout=6m41s&timeoutSeconds=401&watch=true",
            "responseStatus": {"code": 403},
            "sourceIPs": ["5.5.5.5"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-11-29 22:46:38.013",
            "user": {
                "extra": {
                    "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                    "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                },
                "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                "username": "system:serviceaccount:kube-system:coredns",
            },
            "userAgent": "Go-http-client/2.0",
            "verb": "watch",
        },
    ),
    RuleTest(
        name="eks:addon-manager from public ip as lambda",
        expected_result=False,
        log={
            "annotations": {
                "authorization.k8s.io/decision": "allow",
                "authorization.k8s.io/reason": 'RBAC: allowed by RoleBinding "eks:addon-manager/kube-system" of Role "eks:addon-manager" to User "eks:addon-manager"',
            },
            "apiVersion": "audit.k8s.io/v1",
            "auditID": "43410f6e-9c19-482b-b2c7-f2cde260b0e9",
            "kind": "Event",
            "level": "Request",
            "objectRef": {
                "apiGroup": "apps",
                "apiVersion": "v1",
                "name": "coredns",
                "namespace": "kube-system",
                "resource": "deployments",
            },
            "p_any_aws_account_ids": ["123412341234"],
            "p_any_aws_arns": [
                "arn:aws:iam::123412341234:role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF",
                "arn:aws:sts::123412341234:assumed-role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF/1669918824986835422",
            ],
            "p_any_ip_addresses": ["35.163.244.48"],
            "p_any_usernames": ["eks:addon-manager"],
            "p_event_time": "2022-12-01 18:20:25.054",
            "p_log_type": "Amazon.EKS.Audit",
            "p_parse_time": "2022-12-01 18:24:24.734",
            "p_row_id": "a22a2e182591cfb8ead2f7f7149215",
            "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
            "p_source_label": "example-cluster-eks-logs",
            "requestReceivedTimestamp": "2022-12-01 18:20:25.054",
            "requestURI": "/apis/apps/v1/namespaces/kube-system/deployments/coredns?timeout=10s",
            "responseStatus": {"code": 200},
            "sourceIPs": ["35.163.244.48"],
            "stage": "ResponseComplete",
            "stageTimestamp": "2022-12-01 18:20:25.078",
            "user": {
                "extra": {
                    "accessKeyId": ["ASIAXXXXXXXXXXXXXXXX"],
                    "arn": [
                        "arn:aws:sts::123412341234:assumed-role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF/1669918824986835422",
                    ],
                    "canonicalArn": [
                        "arn:aws:iam::123412341234:role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF",
                    ],
                    "sessionName": ["1669918824986835422"],
                },
                "groups": ["system:authenticated"],
                "uid": "aws-iam-authenticator:123412341234:AROATAVZDPHFJWUSNL3ZV",
                "username": "eks:addon-manager",
            },
            "userAgent": "Go-http-client/1.1",
            "verb": "get",
        },
    ),
]


@panther_managed
class AmazonEKSAuditSystemNamespaceFromPublicIP(Rule):
    id = "Amazon.EKS.Audit.SystemNamespaceFromPublicIP-prototype"
    display_name = "EKS Audit Log Reporting system Namespace is Used From A Public IP"
    log_types = [LogType.AMAZON_EKS_AUDIT]
    tags = ["EKS"]
    reports = {"MITRE ATT&CK": ["TA0027:T1475"]}
    default_reference = "https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html"
    default_severity = Severity.INFO
    default_description = 'This detection identifies if an activity is recorded in the Kubernetes audit log where the user:username attribute begins with "system:" or "eks:" and the requests originating IP Address is a Public IP Address\n'
    dedup_period_minutes = 1440
    summary_attributes = ["user:username", "p_source_label"]
    tests = amazon_eks_audit_system_namespace_from_public_ip_tests
    # Explicitly ignore eks:node-manager and eks:addon-manager
    #  which are run as Lambdas and originate from public IPs
    AMZ_PUBLICS = {"eks:addon-manager", "eks:node-manager"}
    # Alert if
    #   the username starts ( with system: or eks: )
    #   and
    #   sourceIPs[0] is a Public Address
    # If not defined, defaults to the rule display name or rule ID.

    def rule(self, event):
        if event.get("stage", "") != "ResponseComplete":
            return False
        # We explicitly ignore 403 here. There is another
        #  detection that monitors for 403 volume-by-originating-ip
        if event.get("responseStatus", {}).get("code", 0) == 403:
            return False
        p_eks = eks_panther_obj_ref(event)
        if (
            p_eks.get("actor") in self.AMZ_PUBLICS
            and ":assumed-role/AWSWesleyClusterManagerLambda"
            in deep_get(event, "user", "extra", "arn", default=["not found"])[0]
        ):
            return False
        if (p_eks.get("actor").startswith("system:") or p_eks.get("actor").startswith("eks:")) and ip_address(
            p_eks.get("sourceIPs")[0],
        ).is_global:
            return True
        return False

    def title(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"[{p_eks.get('actor')}] executed [{p_eks.get('verb')}] for resource [{p_eks.get('resource')}] in ns [{p_eks.get('ns')}] on [{p_eks.get('p_source_label')}] from [{p_eks.get('sourceIPs')[0]}]"

    def dedup(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"{p_eks.get('p_source_label')}_eks_system_namespace_{p_eks.get('sourceIPs')[0]}"

    def alert_context(self, event):
        p_eks = eks_panther_obj_ref(event)
        mutable_event = event.to_dict()
        mutable_event["p_eks"] = p_eks
        return dict(mutable_event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_org_auth_change_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Authentication Method Changed",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "org.saml_disabled",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "org": "my-org",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Non Auth Related Org Change",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "invite_member",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GitHubOrgAuthChange(Rule):
    id = "GitHub.Org.AuthChange-prototype"
    display_name = "GitHub Org Authentication Method Changed"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_severity = Severity.CRITICAL
    summary_attributes = ["actor", "action"]
    default_description = "Detects changes to GitHub org authentication changes."
    default_runbook = "Verify that the GitHub admin performed this activity and validate its use."
    default_reference = (
        "https://docs.github.com/en/authentication/keeping-your-account-and-data-secure/about-authentication-to-github"
    )
    tests = git_hub_org_auth_change_tests
    AUTH_CHANGE_EVENTS = [
        "org.saml_disabled",
        "org.saml_enabled",
        "org.disable_two_factor_requirement",
        "org.enable_two_factor_requirement",
        "org.update_saml_provider_settings",
        "org.enable_oauth_app_restrictions",
        "org.disable_oauth_app_restrictions",
    ]

    def rule(self, event):
        if not event.get("action").startswith("org."):
            return False
        return event.get("action") in self.AUTH_CHANGE_EVENTS

    def title(self, event):
        return f"GitHub auth configuration was changed by {event.get('actor', '<UNKNOWN USER>')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_user_management_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk User Removed",
        expected_result=True,
        log={
            "content": {
                "email": "user@example.com",
                "force": True,
                "name": "user@example.com",
                "userPublicId": "cccccccc-3333-4ddd-8ccc-755555555555",
                "username": "user@example.com",
            },
            "created": "2023-04-11 23:32:14.173",
            "event": "org.user.remove",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk User Invite Revoke",
        expected_result=True,
        log={
            "content": {},
            "created": "2023-04-11 23:32:13.248",
            "event": "org.user.invite.revoke",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group User add",
        expected_result=True,
        log={
            "content": {
                "role": "Group Member",
                "rolePublicId": "65555555-c000-4ddd-2222-cfffffffffff",
                "userPublicId": "cccccccc-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-04-11 23:14:55.572",
            "event": "group.user.add",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk System SSO Setting event happened",
        expected_result=False,
        log={
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "event": "group.sso.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "content": {"unknown": "contents"},
        },
    ),
    RuleTest(
        name="SAML User Added",
        expected_result=False,
        log={
            "content": {
                "role": "Org Collaborator",
                "rolePublicId": "beeeeeee-dddd-4444-aaaa-133333333333",
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-06-01 03:14:42.776",
            "event": "org.user.add",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
]


@panther_managed
class SnykUserManagement(Rule):
    id = "Snyk.User.Management-prototype"
    display_name = "Snyk User Management"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.MEDIUM
    default_description = "Detects when Snyk Users are changed\n"
    default_runbook = "These actions in the Snyk Audit logs indicate that a User has been created/deleted/modified.\n"
    default_reference = "https://docs.snyk.io/snyk-admin/manage-users-and-permissions/member-roles"
    summary_attributes = ["event"]
    tests = snyk_user_management_tests
    ACTIONS = [
        "group.user.add",
        "group.user.provision.accept",
        "group.user.provision.create",
        "group.user.provision.delete",
        "group.user.remove",
        "org.user.add",
        "org.user.invite",
        "org.user.invite.accept",
        "org.user.invite.revoke",
        "org.user.invite_link.accept",
        "org.user.invite_link.create",
        "org.user.invite_link.revoke",
        "org.user.leave",
        "org.user.provision.accept",
        "org.user.provision.create",
        "org.user.provision.delete",
        "org.user.remove",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        # for org.user.add/group.user.add via SAML/SCIM
        # the attributes .userId and .content.publicUserId
        # have the same value
        if action.endswith(".user.add"):
            target_user = deep_get(event, "content", "userPublicId", default="<NO_CONTENT_UID>")
            actor = deep_get(event, "userId", default="<NO_USERID>")
            if target_user == actor:
                return False
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[2:]).title()
        return f"Snyk: [{group_or_org}] User [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        return snyk_alert_context(event)

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        role = deep_get(event, "content", "after", "role", default=None)
        if not role and "afterRoleName" in deep_get(event, "content", default={}):
            role = deep_get(event, "content", "afterRoleName", default=None)
        if role == "ADMIN":
            return "CRITICAL"
        return "MEDIUM"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

awss3_bucket_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="An S3 Bucket was deleted",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "errorCode": "",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AAAAAAAAAAAAAAAAAAAAA:user_name",
                "arn": "arn:aws:sts::123456789012:assumed-role/BucketAdministrator/user_name",
                "accountId": "123456789012",
                "accessKeyId": "AAAAAAAAAAAAAAAAAAAAA",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2020-02-14T00:11:28Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AAAAAAAAAAAAAAAAAAAAA",
                        "arn": "arn:aws:iam::123456789012:role/BucketAdministrator",
                        "accountId": "123456789012",
                        "userName": "BucketAdministrator",
                    },
                },
            },
            "eventTime": "2020-02-14T00:43:54Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "DeleteBucket",
            "awsRegion": "us-east-2",
            "sourceIPAddress": "157.130.196.214",
            "userAgent": "[S3Console/0.4, aws-internal/3 aws-sdk-java/1.11.666 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.232-b09 java/1.8.0_232 vendor/Oracle_Corporation]",
            "requestParameters": {"host": ["s3-us-east-2.amazonaws.com"], "bucketName": "secrets"},
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "AuthenticationMethod": "AuthHeader",
                "vpcEndpointId": "vpce-aaa333c9",
            },
            "requestID": "EEEE5AAAAAA44444",
            "eventID": "6795ef5c-7777-4444-8888-cabb7f252bd3",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
            "vpcEndpointId": "vpce-aaa333c9",
        },
    ),
    RuleTest(
        name="S3 Bucket Deletion Failed",
        expected_result=False,
        log={
            "eventName": "DeleteBucket",
            "errorCode": "BucketNotEmpty",
            "errorMessage": "The bucket you tried to delete is not empty",
            "awsRegion": "us-east-2",
            "sourceIPAddress": "157.130.196.214",
            "userAgent": "[S3Console/0.4, aws-internal/3 aws-sdk-java/1.11.666 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.232-b09 java/1.8.0_232 vendor/Oracle_Corporation]",
            "responseElements": None,
            "requestID": "EEEE5AAAAAA44444",
            "userIdentity": {},
            "eventID": "6795ef5c-7777-4444-8888-cabb7f252bd3",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
            "vpcEndpointId": "vpce-aaa333c9",
        },
    ),
    RuleTest(
        name="An S3 Object Deleted",
        expected_result=False,
        log={
            "eventName": "DeleteObject",
            "awsRegion": "us-east-2",
            "sourceIPAddress": "157.130.196.214",
            "userAgent": "[S3Console/0.4, aws-internal/3 aws-sdk-java/1.11.666 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.232-b09 java/1.8.0_232 vendor/Oracle_Corporation]",
            "responseElements": None,
            "requestID": "EEEE5AAAAAA44444",
            "eventID": "6795ef5c-7777-4444-8888-cabb7f252bd3",
            "userIdentity": {},
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
            "vpcEndpointId": "vpce-aaa333c9",
        },
    ),
]


@panther_managed
class AWSS3BucketDeleted(Rule):
    id = "AWS.S3.BucketDeleted-prototype"
    display_name = "S3 Bucket Deleted"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Impact:Data Destruction"]
    reports = {"MITRE ATT&CK": ["TA0040:T1485"]}
    default_severity = Severity.INFO
    default_description = "A S3 Bucket, Policy, or Website was deleted"
    default_runbook = "Explore if this bucket deletion was potentially destructive"
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/userguide/DeletingObjects.html"
    summary_attributes = ["sourceIpAddress", "userAgent", "recipientAccountId", "vpcEndpointId"]
    tests = awss3_bucket_deleted_tests

    def rule(self, event):
        # Capture DeleteBucket, DeleteBucketPolicy, DeleteBucketWebsite
        return event.get("eventName").startswith("DeleteBucket") and aws_cloudtrail_success(event)

    def helper_strip_role_session_id(self, user_identity_arn):
        # The Arn structure is arn:aws:sts::123456789012:assumed-role/RoleName/<sessionId>
        arn_parts = user_identity_arn.split("/")
        if arn_parts:
            return "/".join(arn_parts[:2])
        return user_identity_arn

    def dedup(self, event):
        user_identity = event.get("userIdentity", {})
        if user_identity.get("type") == "AssumedRole":
            return self.helper_strip_role_session_id(user_identity.get("arn", ""))
        return user_identity.get("arn", "<NO_ARN_FOUND>")

    def title(self, event):
        return f"{deep_get(event, 'userIdentity', 'type')} [{self.dedup(event)}] destroyed a bucket"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

one_password_lut_sensitive_item_tests: list[RuleTest] = [
    RuleTest(
        name="1Password - Sensitive Item Accessed",
        expected_result=True,
        log={
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20195",
                "ip_address": "1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "10.15.7",
                "platform_name": "Chrome",
                "platform_version": "98.0.4758.102",
            },
            "item_uuid": "1234",
            "p_enrichment": {
                "1Password Translation": {
                    "item_uuid": {
                        "title": "demo_item",
                        "updatedAt": "2022-02-14 17:44:50.000000000",
                        "uuid": "12344321",
                    },
                },
            },
            "p_log_type": "OnePassword.ItemUsage",
            "timestamp": "2022-02-23 22:11:50.591",
            "user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "12345"},
            "uuid": "12345",
            "vault_uuid": "54321",
        },
    ),
    RuleTest(
        name="1Password - Non-Sensitive Item Accessed",
        expected_result=False,
        log={
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20195",
                "ip_address": "1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "10.15.7",
                "platform_name": "Chrome",
                "platform_version": "98.0.4758.102",
            },
            "item_uuid": "1234",
            "p_enrichment": {
                "1Password Translation": {
                    "item_uuid": {
                        "title": "not_sensitive",
                        "updatedAt": "2022-02-14 17:44:50.000000000",
                        "uuid": "12344321",
                    },
                },
            },
            "p_log_type": "OnePassword.ItemUsage",
            "timestamp": "2022-02-23 22:11:50.591",
            "user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "12345"},
            "uuid": "12345",
            "vault_uuid": "54321",
        },
    ),
]


@panther_managed
class OnePasswordLutSensitiveItem(Rule):
    id = "OnePassword.Lut.Sensitive.Item-prototype"
    dedup_period_minutes = 30
    display_name = "BETA - Sensitive 1Password Item Accessed"
    enabled = False
    log_types = [LogType.ONEPASSWORD_ITEM_USAGE]
    default_reference = "https://support.1password.com/1password-com-items/"
    default_severity = Severity.LOW
    default_description = "Alerts when a user defined list of sensitive items in 1Password is accessed"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tags = ["Configuration Required", "1Password", "Lookup Table", "BETA", "Credential Access:Unsecured Credentials"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    tests = one_password_lut_sensitive_item_tests
    "\nThis rule requires the use of the Lookup Table feature currently in Beta in Panther, 1Password\nlogs reference items by their UUID without human-friendly titles. The instructions to create a\nlookup table to do this translation can be found at :\n\n https://docs.runpanther.io/guides/using-lookup-tables-1password-uuids\n\nThe steps detailed in that document are required for this rule to function as intended.\n"
    # Add the human-readable names of 1Password items you want to monitor
    SENSITIVE_ITEM_WATCHLIST = ["demo_item"]

    def rule(self, event):
        return (
            deep_get(event, "p_enrichment", "1Password Translation", "item_uuid", "title")
            in self.SENSITIVE_ITEM_WATCHLIST
        )

    def title(self, event):
        return f"A Sensitive 1Password Item was Accessed by user {deep_get(event, 'user', 'name')}"

    def alert_context(self, event):
        context = {
            "user": deep_get(event, "user", "name"),
            "item_name": deep_get(event, "p_enrichment", "1Password Translation", "item_uuid", "title"),
            "client": deep_get(event, "client", "app_name"),
            "ip_address": event.udm("source_ip"),
            "event_time": event.get("timestamp"),
        }
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_local_user_login_without_mfa_tests: list[RuleTest] = [
    RuleTest(
        name="User logged in with MFA",
        expected_result=False,
        log={
            "addr.remote": "[2001:db8:feed:face:c0ff:eeb0:baf00:00d]:65123",
            "cluster_name": "teleport.example.com",
            "code": "T1000I",
            "ei": 0,
            "event": "user.login",
            "method": "local",
            "mfa_device": {
                "mfa_device_name": "1Password",
                "mfa_device_type": "WebAuthn",
                "mfa_device_uuid": "88888888-4444-4444-4444-222222222222",
            },
            "success": True,
            "time": "2023-09-20T19:00:00.123456Z",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "max.mustermann",
            "user_agent": "Examplecorp Spacedeck-web/99.9 (Hackintosh; ARM Cortex A1000)",
        },
    ),
    RuleTest(
        name="User logged in without MFA",
        expected_result=False,
        log={
            "addr.remote": "[2001:db8:face:face:face:face:face:face]:65123",
            "cluster_name": "teleport.example.com",
            "code": "T1000I",
            "ei": 0,
            "event": "user.login",
            "method": "local",
            "success": True,
            "time": "2023-09-20T19:00:00.123456Z",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "max.mustermann",
            "user_agent": "Examplecorp Spacedeck-web/99.9 (Hackintosh; ARM Cortex A1000)",
        },
    ),
]


@panther_managed
class TeleportLocalUserLoginWithoutMFA(Rule):
    id = "Teleport.LocalUserLoginWithoutMFA-prototype"
    display_name = "User Logged in wihout MFA"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.HIGH
    default_description = "A local User logged in without MFA"
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "A local user logged in without Multi-Factor Authentication\n"
    summary_attributes = ["event", "code", "user", "success", "mfa_device"]
    tests = teleport_local_user_login_without_mfa_tests
    SENSITIVE_LOCAL_USERS = ["breakglass"]

    def rule(self, event):
        return (
            event.get("event") == "user.login"
            and event.get("success") == "true"
            and (event.get("method") == "local")
            and (not event.get("mfa_device"))
        )

    def severity(self, event):
        if event.get("user") in self.SENSITIVE_LOCAL_USERS:
            return "HIGH"
        return "MEDIUM"

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] logged into [{event.get('cluster_name', '<UNNAMED_CLUSTER>')}] locally without using MFA"



import json
from fnmatch import fnmatch

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get, get_binding_deltas

ADMIN_ROLES = {
    # Primitive Rolesx
    "roles/owner",
    # Predefined Roles
    "roles/*Admin",
}


def get_event_type(event):
    # currently, only tracking a handful of event types
    for delta in get_binding_deltas(event):
        if delta["action"] == "ADD":
            if any(fnmatch(delta.get("role", ""), admin_role_pattern) for admin_role_pattern in ADMIN_ROLES):
                return event_type.ADMIN_ROLE_ASSIGNED

    return None


def get_admin_map(event):
    roles_assigned = {}
    for delta in get_binding_deltas(event):
        if delta.get("action") == "ADD":
            roles_assigned[delta.get("member")] = delta.get("role")

    return roles_assigned


def get_modified_users(event):
    event_dict = event.to_dict()
    roles_assigned = get_admin_map(event_dict)

    return json.dumps(list(roles_assigned.keys()))


def get_iam_roles(event):
    event_dict = event.to_dict()
    roles_assigned = get_admin_map(event_dict)

    return json.dumps(list(roles_assigned.values()))


def get_api_group(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    try:
        return deep_get(event, "protoPayload", "resourceName", default="").split("/")[0]
    except IndexError:
        return ""


def get_api_version(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    try:
        return deep_get(event, "protoPayload", "resourceName", default="").split("/")[1]
    except IndexError:
        return ""


def get_namespace(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    try:
        return deep_get(event, "protoPayload", "resourceName", default="").split("/")[3]
    except IndexError:
        return ""


def get_resource(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    try:
        return deep_get(event, "protoPayload", "resourceName", default="").split("/")[4]
    except IndexError:
        return ""


def get_name(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    try:
        return deep_get(event, "protoPayload", "resourceName", default="").split("/")[5]
    except IndexError:
        return ""


def get_request_uri(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    return "/apis/" + deep_get(event, "protoPayload", "resourceName", default="")


def get_source_ips(event):
    caller_ip = deep_get(event, "protoPayload", "requestMetadata", "callerIP", default=None)
    if caller_ip:
        return [caller_ip]
    return []


def get_verb(event):
    if deep_get(event, "protoPayload", "serviceName", default="") != "k8s.io":
        return ""
    return deep_get(event, "protoPayload", "methodName", default="").split(".")[-1]


class StandardGCPAuditLog(DataModel):
    id: str = "Standard.GCP.AuditLog"
    display_name: str = "GCP Audit Log"
    enabled: bool = True
    log_types: list[str] = [LogType.GCP_AUDIT_LOG]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.protoPayload.authenticationInfo.principalEmail"),
        DataModelMapping(name="assigned_admin_role", method=get_iam_roles),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="$.protoPayload.requestMetadata.callerIP"),
        DataModelMapping(name="user", method=get_modified_users),
        DataModelMapping(name="annotations", path="$.labels"),
        DataModelMapping(name="apiGroup", method=get_api_group),
        DataModelMapping(name="apiVersion", method=get_api_version),
        DataModelMapping(name="namespace", method=get_namespace),
        DataModelMapping(name="resource", method=get_resource),
        DataModelMapping(name="name", method=get_name),
        DataModelMapping(name="requestURI", method=get_request_uri),
        DataModelMapping(name="responseStatus", path="$.protoPayload.status"),
        DataModelMapping(name="sourceIPs", method=get_source_ips),
        DataModelMapping(name="username", path="$.protoPayload.authenticationInfo.principalEmail"),
        DataModelMapping(name="userAgent", path="$.protoPayload.requestMetadata.callerSuppliedUserAgent"),
        DataModelMapping(name="verb", method=get_verb),
        DataModelMapping(name="requestObject", path="$.protoPayload.request"),
        DataModelMapping(name="responseObject", path="$.protoPayload.response"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

aws_console_sign_in_tests: list[RuleTest] = [
    RuleTest(
        name="Test-94439c",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "8cb05708-9764-4774-a048-59a4c8e1684d",
            "eventName": "Authenticate",
            "eventSource": "sso.amazonaws.com",
            "eventTime": "2024-06-03 15:23:22.000000000",
            "eventType": "AwsServiceEvent",
            "eventVersion": "1.08",
            "managementEvent": True,
        },
    ),
]


@panther_managed
class AWSConsoleSignIn(Rule):
    id = "AWS.Console.Sign-In-prototype"
    display_name = "SIGNAL - AWS Console SSO Sign-In"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    tests = aws_console_sign_in_tests

    def rule(self, event):
        return event.get("eventSource") == "sso.amazonaws.com" and event.get("eventName") == "Authenticate"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description, duo_alert_context

duo_admin_create_admin_tests: list[RuleTest] = [
    RuleTest(
        name="Admin Create",
        expected_result=True,
        log={
            "action": "admin_create",
            "description": '{"name": "Homer Simpson", "phone": null, "is_temporary_password": false, "email": "homer.simpson@simpsons.com", "hardtoken": null, "role": "Owner", "status": "Pending Activation", "restricted_by_admin_units": false, "administrative_units": ""}',
            "isotimestamp": "2023-01-17 16:47:54",
            "object": "Homer Simpson",
            "timestamp": "2023-01-17 16:47:54",
            "username": "Bart Simpson",
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-123", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-07-02 18:31:25",
            "timestamp": "2021-07-02 18:31:25",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminCreateAdmin(Rule):
    default_description = "A new Duo Administrator was created. "
    display_name = "Duo Admin Create Admin"
    default_reference = "https://duo.com/docs/administration-admins#add-an-administrator"
    default_severity = Severity.HIGH
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.Create.Admin-prototype"
    tests = duo_admin_create_admin_tests

    def rule(self, event):
        return event.get("action") == "admin_create"

    def title(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return f"Duo: [{event.get('username', '<username_not_found>')}] created a new admin account: [{event_description.get('name', '<name_not_found>')}] [{event_description.get('email', '<email_not_found>')}]."

    def alert_context(self, event):
        return duo_alert_context(event)



from ipaddress import ip_network

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsvpc_unapproved_outbound_dns_tests: list[RuleTest] = [
    RuleTest(
        name="Approved Outbound DNS Traffic",
        expected_result=False,
        log={"dstPort": 53, "dstAddr": "1.1.1.1", "srcAddr": "10.0.0.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Unapproved Outbound DNS Traffic",
        expected_result=True,
        log={"dstPort": 53, "dstAddr": "100.100.100.100", "srcAddr": "10.0.0.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Outbound Non-DNS Traffic",
        expected_result=False,
        log={"dstPort": 80, "dstAddr": "100.100.100.100", "srcAddr": "10.0.0.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Approved Outbound DNS Traffic - OCSF",
        expected_result=False,
        log={
            "dst_endpoint": {"ip": "1.1.1.1", "port": 53},
            "src_endpoint": {"ip": "10.0.0.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
    RuleTest(
        name="Unapproved Outbound DNS Traffic - OCSF",
        expected_result=True,
        log={
            "dst_endpoint": {"ip": "100.100.100.100", "port": 53},
            "src_endpoint": {"ip": "10.0.0.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
]


@panther_managed
class AWSVPCUnapprovedOutboundDNS(Rule):
    id = "AWS.VPC.UnapprovedOutboundDNS-prototype"
    display_name = "VPC Flow Logs Unapproved Outbound DNS Traffic"
    enabled = False
    log_types = [LogType.AWS_VPC_FLOW, LogType.OCSF_NETWORK_ACTIVITY]
    tags = [
        "AWS",
        "DataModel",
        "Configuration Required",
        "Security Control",
        "Command and Control:Application Layer Protocol",
    ]
    reports = {"MITRE ATT&CK": ["TA0011:T1071"]}
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html"
    default_severity = Severity.MEDIUM
    default_description = "Alerts if outbound DNS traffic is detected to a non-approved DNS server. DNS is often used as a means to exfiltrate data or perform command and control for compromised hosts. All DNS traffic should be routed through internal DNS servers or trusted 3rd parties.\n"
    default_runbook = "Investigate the host sending unapproved DNS activity for signs of compromise or other malicious activity. Update network configurations appropriately to ensure all DNS traffic is routed to approved DNS servers.\n"
    summary_attributes = ["srcaddr", "dstaddr", "dstport"]
    tests = awsvpc_unapproved_outbound_dns_tests  # CloudFlare DNS
    # Google DNS
    # '10.0.0.1', # Internal DNS
    APPROVED_DNS_SERVERS = {"1.1.1.1", "8.8.8.8"}

    def rule(self, event):
        # Common DNS ports, for better security use an application layer aware network monitor
        #
        # Defaults to True (no alert) if 'dstport' key is not present
        if event.udm("destination_port") != 53 and event.udm("destination_port") != 5353:
            return False
        # Only monitor traffic that is originating internally
        #
        # Defaults to True (no alert) if 'srcaddr' key is not present
        source_ip = event.udm("source_ip") or "0.0.0.0/32"
        if ip_network(source_ip).is_global:
            return False
        # No clean way to default to False (no alert), so explicitly check for key
        return bool(event.udm("destination_ip")) and event.udm("destination_ip") not in self.APPROVED_DNS_SERVERS

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_samlsso_configuration_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            },
        },
    ),
    RuleTest(
        name="SAML SSO Enabled",
        expected_result=True,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.settings.enforce_saml_sso_config_updated",
                "workspace.settings.enforce_saml_sso_config_updated": {"state": "enabled"},
            },
        },
    ),
    RuleTest(
        name="SAML SSO Disabled",
        expected_result=True,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.settings.enforce_saml_sso_config_updated",
                "workspace.settings.enforce_saml_sso_config_updated": {"state": "disabled"},
            },
        },
    ),
]


@panther_managed
class NotionSAMLSSOConfigurationChanged(Rule):
    id = "Notion.SAML.SSO.Configuration.Changed-prototype"
    display_name = "Notion SAML SSO Configuration Changed"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Identity & Access Management", "Credential Security"]
    default_severity = Severity.HIGH
    default_description = "A Notion User changed settings to enforce SAML SSO configurations for your organization."
    default_runbook = "Follow up with the Notion User to determine if this was done for a valid business reason and to ensure these settings get re-enabled quickly for best security practices."
    default_reference = "https://www.notion.so/help/saml-sso-configuration"
    tests = notion_samlsso_configuration_changed_tests

    def rule(self, event):
        return (
            event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
            == "workspace.settings.enforce_saml_sso_config_updated"
        )

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        state = deep_get(
            event,
            "event",
            "workspace.settings.enforce_saml_sso_config_updated",
            "state",
            default="<NO_STATE_FOUND>",
        )
        if state == "enabled":
            return f"Notion User [{user}] updated settings to enable SAML SSO config from workspace id {workspace_id}"
        return f"Notion User [{user}] updated settings to disable SAML SSO config from workspace id {workspace_id}"

    def severity(self, event):
        state = deep_get(
            event,
            "event",
            "workspace.settings.enforce_saml_sso_config_updated",
            "state",
            default="<NO_STATE_FOUND>",
        )
        if state == "enabled":
            return "INFO"
        return "HIGH"

    def alert_context(self, event):
        return notion_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_software_discovery_tests: list[RuleTest] = [
    RuleTest(
        name="Discovery Event Names",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "6faccad9-b6ae-4549-8e39-03430cbce2aa",
            "eventName": "DescribeSecurityGroups",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2021-10-19 01:06:59",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["853509373758"],
            "p_any_aws_arns": [
                "arn:aws:iam::853509373758:role/PantherAuditRole-us-east-2",
                "arn:aws:sts::853509373758:assumed-role/PantherAuditRole-us-east-2/1634605518650090138",
            ],
            "p_any_ip_addresses": ["3.16.40.237"],
            "p_event_time": "2021-10-19 01:06:59",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2021-10-19 01:11:10.412",
            "p_row_id": "eabaceda7842c2b2e7a398f40cc150",
            "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
            "p_source_label": "Hosted - Cloudtrail - KoS",
            "readOnly": True,
            "recipientAccountId": "853509373758",
            "requestID": "43efad11-bb40-43df-ad25-c8e7f0bfdc7a",
            "requestParameters": {
                "filterSet": {},
                "securityGroupIdSet": {"items": [{"groupId": "sg-01e29ae063f5f63a0"}]},
                "securityGroupSet": {},
            },
            "sourceIPAddress": "3.16.40.237",
            "userAgent": "aws-sdk-go/1.40.21 (go1.17; linux; amd64) exec-env/AWS_Lambda_go1.x",
            "userIdentity": {
                "accessKeyId": "ASIA4NOI7P47DJPFAXF2",
                "accountId": "853509373758",
                "arn": "arn:aws:sts::853509373758:assumed-role/PantherAuditRole-us-east-2/1634605518650090138",
                "principalId": "AROA4NOI7P47OHH3NQORX:1634605518650090138",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-19T01:05:18Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "853509373758",
                        "arn": "arn:aws:iam::853509373758:role/PantherAuditRole-us-east-2",
                        "principalId": "AROA4NOI7P47OHH3NQORX",
                        "type": "Role",
                        "userName": "PantherAuditRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Non Discovery Event Names",
        expected_result=False,
        log={
            "apiVersion": "2012-08-10",
            "awsRegion": "us-east-1",
            "eventCategory": "Data",
            "eventID": "9b666120-d5f9-4ca8-b158-3cd53e4c48bd",
            "eventName": "GetRecords",
            "eventSource": "dynamodb.amazonaws.com",
            "eventTime": "2023-01-09 16:01:21",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": False,
        },
    ),
]


@panther_managed
class AWSSoftwareDiscovery(Rule):
    default_description = (
        "A user is obtaining a list of security software, configurations, defensive tools, and sensors that are in AWS."
    )
    display_name = "AWS Software Discovery"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1518/001/"
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0007:T1518"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 360
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Software.Discovery-prototype"
    threshold = 50
    tests = aws_software_discovery_tests
    DISCOVERY_EVENTS = [
        "ListDocuments",
        "ListMembers",
        "DescribeProducts",
        "DescribeStandards",
        "DescribeStandardsControls",
        "DescribeInstanceInformation",
        "DescribeSecurityGroups",
        "DescribeSecurityGroupRules",
        "DescribeSecurityGroupReferences",
        "DescribeSubnets",
        "DescribeHub",
        "ListFirewalls",
        "ListRuleGroups",
        "ListFirewallPolicies",
        "DescribeFirewall",
        "DescribeFirewallPolicy",
        "DescribeLoggingConfiguration",
        "DescribeResourcePolicy",
        "DescribeRuleGroup",
    ]

    def rule(self, event):
        return event.get("eventName") in self.DISCOVERY_EVENTS

    def title(self, event):
        return f"User [{deep_get(event, 'userIdentity', 'principalId')}] performed a [{event.get('eventName')}] action in AWS account [{event.get('recipientAccountId')}]."

    def dedup(self, event):
        return deep_get(event, "userIdentity", "principalId", default="NO_PRINCIPAL_ID_FOUND")

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsec2_ebs_encryption_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="DisableEbsEncryptionByDefault Event",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventName": "DisableEbsEncryptionByDefault",
            "eventSource": "ec2.amazonaws.com",
            "recipientAccountId": "123456789",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "Chrome Browser",
        },
    ),
    RuleTest(
        name="Non Matching Event",
        expected_result=False,
        log={
            "awsRegion": "ap-northeast-1",
            "eventName": "DescribeInstanceStatus",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-09-25 16:16:37",
            "eventType": "AwsApiCall",
            "readOnly": True,
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "Datadog",
        },
    ),
]


@panther_managed
class AWSEC2EBSEncryptionDisabled(Rule):
    default_description = "Identifies disabling of default EBS encryption. Disabling default encryption does not change the encryption status of existing volumes. "
    display_name = "AWS EC2 EBS Encryption Disabled"
    reports = {"MITRE ATT&CK": ["TA0040:T1486", "TA0040:T1565"]}
    default_runbook = "Verify this action was intended and if any EBS volumes were created after the change."
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default"
    default_severity = Severity.MEDIUM
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.EBS.Encryption.Disabled-prototype"
    tests = awsec2_ebs_encryption_disabled_tests

    def rule(self, event):
        return (
            event.get("eventSource") == "ec2.amazonaws.com"
            and event.get("eventName") == "DisableEbsEncryptionByDefault"
        )

    def title(self, event):
        return f"EC2 EBS Default Encryption was disabled in [{event.get('recipientAccountId')}] - [{event.get('awsRegion')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_secret_scanning_alert_created_tests: list[RuleTest] = [
    RuleTest(
        name="secret_scanning_alert.create-true",
        expected_result=True,
        log={
            "action": "secret_scanning_alert.create",
            "actor": "github",
            "actor_id": "1234",
            "business": "Acme Inc.",
            "business_id": "12345",
            "created_at": "2023-10-18 18:20:52.209000000",
            "number": 12,
            "org": "acme-inc",
            "org_id": 1234567,
            "repo": "acme-inc/crown-jewels",
            "repo_id": 123456789,
        },
    ),
    RuleTest(
        name="git.clone-false",
        expected_result=False,
        log={
            "_document_id": "KCYtigpnShPBSohA4OXbRg==",
            "action": "git.clone",
            "actor": "acme-inc-user",
            "actor_id": "123456789",
            "actor_ip": "5.6.7.8",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2023-11-20 21:20:09.423",
            "business": "acme-inc",
            "business_id": "12345",
            "external_identity_nameid": "",
            "external_identity_username": "",
            "hashed_token": "0771cae4d170dc02a6ff393d9946cd684d6145c1=",
            "org": "acme-inc",
            "org_id": 12345678,
            "programmatic_access_type": "OAuth access token",
            "repo": "acme-inc/a-repo",
            "repository": "acme-inc/a-repo",
            "repository_public": False,
            "token_id": "0123456789",
            "transport_protocol": 1,
            "transport_protocol_name": "http",
            "user": "",
            "user_agent": "git/2.30.2",
            "user_id": "0",
        },
    ),
]


@panther_managed
class GitHubSecretScanningAlertCreated(Rule):
    id = "GitHub.Secret.Scanning.Alert.Created-prototype"
    display_name = "GitHub Secret Scanning Alert Created"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "GitHub detected a secret and created a secret scanning alert."
    default_runbook = "Review the secret to determine if it needs to be revoked or the alert suppressed."
    default_reference = "https://docs.github.com/en/code-security/secret-scanning/about-secret-scanning"
    tests = git_hub_secret_scanning_alert_created_tests

    def rule(self, event):
        return event.get("action", "") == "secret_scanning_alert.create"

    def title(self, event):
        return f"Github detected a secret in {event.get('repo', '<REPO_NOT_FOUND>')} (#{event.get('number', '<NUMBER_NOT_FOUND>')})"

    def alert_context(self, event):
        return {
            "github_organization": event.get("org", "<ORG_NOT_FOUND>"),
            "github_repository": event.get("repo", "<REPO_NOT_FOUND>"),
            "alert_number": str(event.get("number", "<NUMBER_NOT_FOUND>")),
            "url": f"https://github.com/{event.get('repo')}/security/secret-scanning/{event.get('number')}"
            if all([event.get("repo"), event.get("number")])
            else "<URL_NOT_FOUND>",
        }



import json

from policyuniverse.policy import Policy

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_resource_made_public_tests: list[RuleTest] = [
    RuleTest(
        name="ECR Made Public",
        expected_result=True,
        log={
            "awsRegion": "eu-west-1",
            "eventID": "685e066d-a3aa-4323-a6a1-2f187a2fc986",
            "eventName": "SetRepositoryPolicy",
            "eventSource": "ecr.amazonaws.com",
            "eventTime": "2020-11-20 06:19:05.000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "112233445566",
            "requestID": "95fd6392-627c-467b-b940-895183d3298d",
            "requestParameters": {
                "force": False,
                "policyText": '{"Version":"2012-10-17","Statement":[{"Action":["ecr:BatchCheckLayerAvailability","ecr:BatchGetImage","ecr:GetAuthorizationToken","ecr:GetDownloadUrlForLayer"],"Effect":"Allow","Principal":"*","Sid":"PublicRead"}]}',
                "repositoryName": "community",
            },
            "resources": [
                {"accountId": "112233445566", "arn": "arn:aws:ecr:eu-west-1:112233445566:repository/community"},
            ],
            "responseElements": {
                "policyText": '{\n  "Version" : "2012-10-17",\n  "Statement" : [ {\n    "Sid" : "PublicRead",\n    "Effect" : "Allow",\n    "Principal" : "*",\n    "Action" : [ "ecr:BatchCheckLayerAvailability", "ecr:BatchGetImage", "ecr:GetAuthorizationToken", "ecr:GetDownloadUrlForLayer" ]\n  } ]\n}',
                "registryId": "112233445566",
                "repositoryName": "community",
            },
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAIJJG73VC6IW5OFVQ",
                "accountId": "112233445566",
                "arn": "arn:aws:sts::112233445566:assumed-role/ServiceRole/AWSCloudFormation",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAJJJJTTTT44445IJJJ:AWSCloudFormation",
                "sessionContext": {
                    "attributes": {"creationDate": "2020-11-20T06:19:04Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "112233445566",
                        "arn": "arn:aws:iam::112233445566:role/ServiceRole",
                        "principalId": "AROAJJJJTTTT44445IJJJ",
                        "type": "Role",
                        "userName": "ServiceRole",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
            "p_event_time": "2020-11-20 06:19:05.000",
            "p_parse_time": "2020-11-20 06:31:53.258",
            "p_log_type": "AWS.CloudTrail",
            "p_row_id": "ea68a92f0295a6bed49fa8af068faa05",
            "p_any_aws_account_ids": ["112233445566"],
            "p_any_aws_arns": [
                "arn:aws:ecr:eu-west-1:112233445566:repository/community",
                "arn:aws:iam::112233445566:role/ServiceRole",
                "arn:aws:sts::112233445566:assumed-role/ServiceRole/AWSCloudFormation",
            ],
        },
    ),
    RuleTest(
        name="S3 Made Publicly Accessible",
        expected_result=True,
        log={
            "additionalEventData": {
                "AuthenticationMethod": "AuthHeader",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "SignatureVersion": "SigV4",
                "vpcEndpointId": "vpce-1111",
            },
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutBucketPolicy",
            "eventSource": "s3.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "bucketName": "example-bucket",
                "bucketPolicy": {
                    "Statement": [
                        {
                            "Action": "s3:GetBucketAcl",
                            "Effect": "Allow",
                            "Principal": {"AWS": "*"},
                            "Resource": "arn:aws:s3:::example-bucket",
                            "Sid": "Public Access",
                        },
                    ],
                    "Version": "2012-10-17",
                },
                "host": ["s3.us-west-2.amazonaws.com"],
                "policy": [""],
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                },
                "type": "AssumedRole",
            },
            "vpcEndpointId": "vpce-1111",
        },
    ),
    RuleTest(
        name="S3 Not Made Publicly Accessible",
        expected_result=False,
        log={
            "additionalEventData": {
                "AuthenticationMethod": "AuthHeader",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "SignatureVersion": "SigV4",
                "vpcEndpointId": "vpce-1111",
            },
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutBucketPolicy",
            "eventSource": "s3.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "bucketName": "example-bucket",
                "bucketPolicy": {
                    "Statement": [
                        {
                            "Action": "s3:GetBucketAcl",
                            "Effect": "Allow",
                            "Principal": {"Service": "cloudtrail.amazonaws.com"},
                            "Resource": "arn:aws:s3:::example-bucket",
                            "Sid": "Public Access",
                        },
                    ],
                    "Version": "2012-10-17",
                },
                "host": ["s3.us-west-2.amazonaws.com"],
                "policy": [""],
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                },
                "type": "AssumedRole",
            },
            "vpcEndpointId": "vpce-1111",
        },
    ),
    RuleTest(
        name="Null Request Parameters",
        expected_result=False,
        log={
            "additionalEventData": {
                "AuthenticationMethod": "AuthHeader",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "SignatureVersion": "SigV4",
                "vpcEndpointId": "vpce-1111",
            },
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutBucketPolicy",
            "eventSource": "s3.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": None,
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                },
                "type": "AssumedRole",
            },
            "vpcEndpointId": "vpce-1111",
        },
    ),
    RuleTest(
        name="S3 Failed to make Publicly Accessible",
        expected_result=False,
        log={
            "additionalEventData": {
                "AuthenticationMethod": "AuthHeader",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "SignatureVersion": "SigV4",
                "vpcEndpointId": "vpce-1111",
            },
            "errorCode": "AccessDenied",
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutBucketPolicy",
            "eventSource": "s3.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "bucketName": "example-bucket",
                "bucketPolicy": {
                    "Statement": [
                        {
                            "Action": "s3:GetBucketAcl",
                            "Effect": "Allow",
                            "Principal": {"AWS": "*"},
                            "Resource": "arn:aws:s3:::example-bucket",
                            "Sid": "Public Access",
                        },
                    ],
                    "Version": "2012-10-17",
                },
                "host": ["s3.us-west-2.amazonaws.com"],
                "policy": [""],
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                },
                "type": "AssumedRole",
            },
            "vpcEndpointId": "vpce-1111",
        },
    ),
    RuleTest(
        name="Empty Policy Payload",
        expected_result=False,
        log={
            "additionalEventData": {
                "AuthenticationMethod": "AuthHeader",
                "CipherSuite": "ECDHE-RSA-AES128-SHA",
                "SignatureVersion": "SigV4",
                "vpcEndpointId": "vpce-1111",
            },
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "SetQueueAttributes",
            "eventSource": "s3.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributes": {"Policy": ""},
                "queueUrl": "https://sqs.us-east-1.amazonaws.com/123456789012/example-queue",
            },
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                },
                "type": "AssumedRole",
            },
            "vpcEndpointId": "vpce-1111",
        },
    ),
]


@panther_managed
class AWSCloudTrailResourceMadePublic(Rule):
    id = "AWS.CloudTrail.ResourceMadePublic-prototype"
    display_name = "AWS Resource Made Public"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration:Transfer Data to Cloud Account"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = "Some AWS resource was made publicly accessible over the internet. Checks ECR, Elasticsearch, KMS, S3, S3 Glacier, SNS, SQS, and Secrets Manager.\n"
    default_runbook = "Adjust the policy so that the resource is no longer publicly accessible"
    default_reference = "https://aws.amazon.com/blogs/security/identifying-publicly-accessible-resources-with-amazon-vpc-network-access-analyzer/"
    summary_attributes = ["userAgent", "sourceIpAddress", "vpcEndpointId", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_resource_made_public_tests
    # Check that the IAM policy allows resource accessibility via the Internet
    # Normally this check helps avoid overly complex functions that are doing too many things,
    # but in this case we explicitly want to handle 10 different cases in 10 different ways.
    # Any solution that avoids too many return statements only increases the complexity of this rule.
    # pylint: disable=too-many-return-statements, too-complex

    def policy_is_internet_accessible(self, json_policy):
        if json_policy is None:
            return False
        return Policy(json_policy).is_internet_accessible()

    def rule(self, event):
        if not aws_cloudtrail_success(event):
            return False
        parameters = event.get("requestParameters", {})
        # Ignore events that are missing request params
        if not parameters:
            return False
        policy = ""
        # S3
        if event["eventName"] == "PutBucketPolicy":
            return self.policy_is_internet_accessible(parameters.get("bucketPolicy"))
        # ECR
        if event["eventName"] == "SetRepositoryPolicy":
            policy = parameters.get("policyText", {})
        # Elasticsearch
        if event["eventName"] in ["CreateElasticsearchDomain", "UpdateElasticsearchDomainConfig"]:
            policy = parameters.get("accessPolicies", {})
        # KMS
        if event["eventName"] in ["CreateKey", "PutKeyPolicy"]:
            policy = parameters.get("policy", {})
        # S3 Glacier
        if event["eventName"] == "SetVaultAccessPolicy":
            policy = deep_get(parameters, "policy", "policy", default={})
        # SNS & SQS
        if event["eventName"] in ["SetQueueAttributes", "CreateTopic"]:
            policy = deep_get(parameters, "attributes", "Policy", default={})
        # SNS
        if event["eventName"] == "SetTopicAttributes" and parameters.get("attributeName", "") == "Policy":
            policy = parameters.get("attributeValue", {})
        # SecretsManager
        if event["eventName"] == "PutResourcePolicy":
            policy = parameters.get("resourcePolicy", {})
        if not policy:
            return False
        return self.policy_is_internet_accessible(json.loads(policy))

    def title(self, event):
        # TODO(): Update this rule to use data models
        user = deep_get(event, "userIdentity", "userName") or deep_get(
            event,
            "userIdentity",
            "sessionContext",
            "sessionIssuer",
            "userName",
            default="<MISSING_USER>",
        )
        if event.get("Resources"):
            return f"Resource {event.get('Resources')[0].get('arn', 'MISSING')} made public by {user}"
        return f"{event.get('eventSource', 'MISSING SOURCE')} resource made public by {user}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_two_step_verification_tests: list[RuleTest] = [
    RuleTest(
        name="Two Step Verification Enabled",
        expected_result=False,
        log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
            "kind": "admin#reports#activity",
            "type": "2sv_change",
            "name": "2sv_enroll",
        },
    ),
    RuleTest(
        name="Two Step Verification Disabled",
        expected_result=True,
        log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
            "kind": "admin#reports#activity",
            "type": "2sv_change",
            "name": "2sv_disable",
        },
    ),
]


@panther_managed
class GSuiteTwoStepVerification(Rule):
    id = "GSuite.TwoStepVerification-prototype"
    display_name = "GSuite User Two Step Verification Change"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_severity = Severity.LOW
    default_description = "A user disabled two step verification for themselves.\n"
    default_reference = (
        "https://support.google.com/mail/answer/185839?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    )
    default_runbook = (
        "Depending on company policy, either suggest or require the user re-enable two step verification.\n"
    )
    summary_attributes = ["actor:email"]
    tests = g_suite_two_step_verification_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "user_accounts":
            return False
        if event.get("type") == "2sv_change" and event.get("name") == "2sv_disable":
            return True
        return False

    def title(self, event):
        return f"Two step verification was disabled for user [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gc_pcomputeinstancescreate_privilege_escalation_tests: list[RuleTest] = [
    RuleTest(
        name="GCP compute.instances - Potential Privilege Escalation",
        expected_result=True,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.instances.create",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.disks.create",
                        "resource": "projects/some-project/zones/us-central1-f/disks/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/disks/abc",
                            "service": "compute",
                            "type": "compute.disks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.use",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.useExternalIp",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.instances.setMetadata",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.instances.setServiceAccount",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                ],
                "methodName": "v1.compute.instances.insert",
                "request": {
                    "@type": "type.googleapis.com/compute.instances.insert",
                    "disks": "...",
                    "machineType": "...",
                    "name": "...",
                    "networkInterfaces": "...",
                    "serviceAccounts": [
                        {
                            "email": "abcmail@some-project.iam.gserviceaccount.com",
                            "scopes": [
                                "https://www.googleapis.com/auth/cloud-platform",
                                "https://www.googleapis.com/auth/iam",
                            ],
                        },
                    ],
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2024-01-30T12:52:36.003867Z"},
                },
                "resourceLocation": "...",
                "resourceName": "projects/some-project/zones/us-central1-f/instances/abc",
                "response": {
                    "@type": "type.googleapis.com/operation",
                    "id": "8758546889396539388",
                    "insertTime": "2024-01-30T04:52:35.886-08:00",
                    "name": "operation-1706619154623-610293c7a6a25-934f1c35-1efebb12",
                    "operationType": "insert",
                    "progress": "0",
                    "selfLink": "https://www.googleapis.com/compute/v1/projects/some-project/zones/us-central1-f/operations/operation-1706619154623-610293c7a6a25-934f1c35-1efebb12",
                    "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/some-project/zones/us-central1-f/operations/8758546889396539388",
                    "startTime": "2024-01-30T04:52:35.887-08:00",
                    "status": "RUNNING",
                    "targetId": "1454427709413609468",
                    "targetLink": "https://www.googleapis.com/compute/v1/projects/some-project/zones/us-central1-f/instances/abc",
                    "user": "some.user@company.com",
                    "zone": "https://www.googleapis.com/compute/v1/projects/some-project/zones/us-central1-f",
                },
                "serviceName": "compute.googleapis.com",
            },
            "receiveTimestamp": "2024-01-30 12:52:36.642422049",
            "resource": {
                "labels": {"instance_id": "1454427709413609468", "project_id": "some-project", "zone": "us-central1-f"},
                "type": "gce_instance",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-30 12:52:34.676384000",
        },
    ),
    RuleTest(
        name="GCP compute.instances - Error",
        expected_result=False,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.instances.create",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.disks.create",
                        "resource": "projects/some-project/zones/us-central1-f/disks/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/disks/abc",
                            "service": "compute",
                            "type": "compute.disks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.use",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.useExternalIp",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.instances.setMetadata",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.instances.setServiceAccount",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                ],
                "methodName": "v1.compute.instances.insert",
                "request": {
                    "@type": "...",
                    "disks": "...",
                    "machineType": "...",
                    "name": "...",
                    "networkInterfaces": "...",
                    "serviceAccounts": [
                        {
                            "email": "abcmail@some-project.iam.gserviceaccount.com",
                            "scopes": [
                                "https://www.googleapis.com/auth/cloud-platform",
                                "https://www.googleapis.com/auth/iam",
                            ],
                        },
                    ],
                },
                "requestMetadata": "...",
                "resourceLocation": "...",
                "resourceName": "projects/some-project/zones/us-central1-f/instances/abc",
                "response": {
                    "@type": "type.googleapis.com/error",
                    "error": {
                        "code": 404,
                        "errors": [
                            {"domain": "global", "message": "The resource 'abc' was not found", "reason": "notFound"},
                        ],
                        "message": "The resource 'abc' was not found",
                    },
                },
                "serviceName": "compute.googleapis.com",
                "status": {"code": 5, "message": "The resource 'abc' was not found"},
            },
            "receiveTimestamp": "2024-01-30 11:03:56.719662927",
            "resource": {
                "labels": {"instance_id": "", "project_id": "some-project", "zone": "us-central1-f"},
                "type": "gce_instance",
            },
            "severity": "ERROR",
            "timestamp": "2024-01-30 11:03:55.700872000",
        },
    ),
    RuleTest(
        name="GCP compute.instances - Not All Permissions",
        expected_result=False,
        log={
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "compute.instances.create",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.disks.create",
                        "resource": "projects/some-project/zones/us-central1-f/disks/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/disks/abc",
                            "service": "compute",
                            "type": "compute.disks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.use",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.subnetworks.useExternalIp",
                        "resource": "projects/some-project/regions/us-central1/subnetworks/default",
                        "resourceAttributes": {
                            "name": "projects/some-project/regions/us-central1/subnetworks/default",
                            "service": "compute",
                            "type": "compute.subnetworks",
                        },
                    },
                    {
                        "granted": True,
                        "permission": "compute.instances.setMetadata",
                        "resource": "projects/some-project/zones/us-central1-f/instances/abc",
                        "resourceAttributes": {
                            "name": "projects/some-project/zones/us-central1-f/instances/abc",
                            "service": "compute",
                            "type": "compute.instances",
                        },
                    },
                ],
                "methodName": "v1.compute.instances.insert",
                "request": "...",
                "requestMetadata": "...",
                "resourceLocation": "...",
                "resourceName": "...",
                "response": "...",
                "serviceName": "...",
            },
            "receiveTimestamp": "...",
            "resource": "...",
        },
    ),
]


@panther_managed
class GCPcomputeinstancescreatePrivilegeEscalation(Rule):
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "Detects compute.instances.create method for privilege escalation in GCP."
    display_name = "GCP compute.instances.create Privilege Escalation"
    id = "GCP.compute.instances.create.Privilege.Escalation-prototype"
    default_reference = "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"
    default_runbook = "Confirm this was authorized and necessary behavior."
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.HIGH
    tests = gc_pcomputeinstancescreate_privilege_escalation_tests
    REQUIRED_PERMISSIONS = [
        "compute.disks.create",
        "compute.instances.create",
        "compute.instances.setMetadata",
        "compute.instances.setServiceAccount",
        "compute.subnetworks.use",
        "compute.subnetworks.useExternalIp",
    ]

    def rule(self, event):
        if deep_get(event, "protoPayload", "response", "error"):
            return False
        method = deep_get(event, "protoPayload", "methodName", default="METHOD_NOT_FOUND")
        if not method.endswith("compute.instances.insert"):
            return False
        authorization_info = deep_get(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        granted_permissions = {}
        for auth in authorization_info:
            granted_permissions[auth["permission"]] = auth["granted"]
        for permission in self.REQUIRED_PERMISSIONS:
            if not granted_permissions.get(permission):
                return False
        return True

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        service_accounts = deep_get(event, "protoPayload", "request", "serviceAccounts")
        if not service_accounts:
            service_account_emails = "<SERVICE_ACCOUNT_EMAILS_NOT_FOUND>"
        else:
            service_account_emails = [service_acc["email"] for service_acc in service_accounts]
        project = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created a new Compute Engine instance with [{service_account_emails}] Service Account on project [{project}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        service_accounts = deep_get(event, "protoPayload", "request", "serviceAccounts")
        if not service_accounts:
            service_account_emails = "<SERVICE_ACCOUNT_EMAILS_NOT_FOUND>"
        else:
            service_account_emails = [service_acc["email"] for service_acc in service_accounts]
        context["serviceAccount"] = service_account_emails
        return context



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config

dropbox_external_share_tests: list[RuleTest] = [
    RuleTest(
        name="Domain in Allowlist",
        expected_result=False,
        mocks=[RuleMock(object_name="DROPBOX_ALLOWED_SHARE_DOMAINS", return_value='[\n    "example.com"\n]')],
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAACjvKy90uezyOiLRadIuCy66dK5d1vGGw",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.com",
                    "team_member_id": "dbmid:AADSERs2cAsByYt8yQEDU4_qdNQiSdxgCl8",
                },
            },
            "assets": [
                {
                    ".tag": "file",
                    "display_name": "paper1.paper",
                    "file_id": "id:lUP4ZxMYmc4AAAAAAAAAaA",
                    "path": {
                        "contextual": "/pathtest/paper1.paper",
                        "namespace_relative": {
                            "is_shared_namespace": True,
                            "ns_id": "3590048721",
                            "relative_path": "/paper1.paper",
                        },
                    },
                },
            ],
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAACjvKy90uezyOiLRadIuCy66dK5d1vGGw",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.com",
                "team_member_id": "dbmid:AADSERs2cAsByYt8yQEDU4_qdNQiSdxgCl8",
            },
            "details": {".tag": "shared_content_add_member_details", "shared_content_access_level": {".tag": "viewer"}},
            "event_category": {"_tag": "sharing"},
            "event_type": {
                "_tag": "shared_content_add_member",
                "description": "Added users and/or groups to shared file/folder",
            },
            "involve_non_team_member": True,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["david.davidson@example.com", "alice.bob@company.com"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob", "david davidson"],
            "p_event_time": "2023-04-18 22:31:03",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 22:32:46.967",
            "p_row_id": "fe2163f14b45f3c1b9a49fd31799a504",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "participants": [
                {
                    "user": {
                        "_tag": "non_team_member",
                        "account_id": "dbid:AABbWylBrTJ3Je-M37jeWShWuMAFHchEsKM",
                        "display_name": "david davidson",
                        "email": "david.davidson@example.com",
                    },
                },
            ],
            "timestamp": "2023-04-18 22:31:03",
        },
    ),
    RuleTest(
        name="external share",
        expected_result=True,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAACjvKy90uezyOiLRadIuCy66dK5d1vGGw",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.com",
                    "team_member_id": "dbmid:AADSERs2cAsByYt8yQEDU4_qdNQiSdxgCl8",
                },
            },
            "assets": [
                {
                    ".tag": "file",
                    "display_name": "paper1.paper",
                    "file_id": "id:lUP4ZxMYmc4AAAAAAAAAaA",
                    "path": {
                        "contextual": "/pathtest/paper1.paper",
                        "namespace_relative": {
                            "is_shared_namespace": True,
                            "ns_id": "3590048721",
                            "relative_path": "/paper1.paper",
                        },
                    },
                },
            ],
            "context": {
                "_tag": "team_member",
                "account_id": "dbid:AAACjvKy90uezyOiLRadIuCy66dK5d1vGGw",
                "display_name": "Alice Bob",
                "email": "alice.bob@company.com",
                "team_member_id": "dbmid:AADSERs2cAsByYt8yQEDU4_qdNQiSdxgCl8",
            },
            "details": {".tag": "shared_content_add_member_details", "shared_content_access_level": {".tag": "viewer"}},
            "event_category": {"_tag": "sharing"},
            "event_type": {
                "_tag": "shared_content_add_member",
                "description": "Added users and/or groups to shared file/folder",
            },
            "involve_non_team_member": True,
            "origin": {
                "access_method": {
                    ".tag": "end_user",
                    "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                },
                "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
            },
            "p_any_emails": ["david.davidson@david.co", "alice.bob@company.com"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_usernames": ["Alice Bob", "david davidson"],
            "p_event_time": "2023-04-18 22:31:03",
            "p_log_type": "Dropbox.TeamEvent",
            "p_parse_time": "2023-04-18 22:32:46.967",
            "p_row_id": "fe2163f14b45f3c1b9a49fd31799a504",
            "p_schema_version": 0,
            "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
            "p_source_label": "dropbox1",
            "participants": [
                {
                    "user": {
                        "_tag": "non_team_member",
                        "account_id": "dbid:AABbWylBrTJ3Je-M37jeWShWuMAFHchEsKM",
                        "display_name": "david davidson",
                        "email": "david.davidson@david.co",
                    },
                },
            ],
            "timestamp": "2023-04-18 22:31:03",
        },
    ),
]


@panther_managed
class DropboxExternalShare(Rule):
    default_description = "Dropbox item shared externally"
    display_name = "Dropbox External Share"
    default_reference = "https://help.dropbox.com/share/share-outside-dropbox"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.External.Share-prototype"
    tests = dropbox_external_share_tests
    DROPBOX_ALLOWED_SHARE_DOMAINS = config.DROPBOX_ALLOWED_SHARE_DOMAINS

    def rule(self, event):
        if isinstance(self.DROPBOX_ALLOWED_SHARE_DOMAINS, MagicMock):
            self.DROPBOX_ALLOWED_SHARE_DOMAINS = set(json.loads(self.DROPBOX_ALLOWED_SHARE_DOMAINS()))  # pylint: disable=not-callable
        if deep_get(event, "event_type", "_tag", default="") == "shared_content_add_member":
            participants = event.get("participants", [{}])
            for participant in participants:
                email = participant.get("user", {}).get("email", "")
                if email.split("@")[-1] not in self.DROPBOX_ALLOWED_SHARE_DOMAINS:
                    return True
        return False

    def title(self, event):
        actor = deep_get(event, "actor", "user", "email", default="<ACTOR_NOT_FOUND>")
        assets = [e.get("display_name", "") for e in event.get("assets", [{}])]
        participants = event.get("participants", [{}])
        external_participants = []
        for participant in participants:
            email = participant.get("user", {}).get("email", "")
            if email.split("@")[-1] not in self.DROPBOX_ALLOWED_SHARE_DOMAINS:
                external_participants.append(email)
        return f"Dropbox: [{actor}] shared [{assets}] with external user [{external_participants}]."

    def alert_context(self, event):
        external_participants = []
        participants = event.get("participants", [{}])
        for participant in participants:
            email = participant.get("user", {}).get("email", "")
            if email.split("@")[-1] not in self.DROPBOX_ALLOWED_SHARE_DOMAINS:
                external_participants.append(email)
        return {"external_participants": external_participants}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config

g_suite_external_mail_forwarding_tests: list[RuleTest] = [
    RuleTest(
        name="Forwarding to External Address",
        expected_result=True,
        log={
            "id": {"applicationName": "user_accounts", "customerId": "D12345"},
            "actor": {"email": "homer.simpson@.springfield.io"},
            "type": "email_forwarding_change",
            "name": "email_forwarding_out_of_domain",
            "parameters": {"email_forwarding_destination_address": "HSimpson@gmail.com"},
        },
    ),
    RuleTest(
        name="Forwarding to External Address - Allowed Domain",
        expected_result=False,
        log={
            "id": {"applicationName": "user_accounts", "customerId": "D12345"},
            "actor": {"email": "homer.simpson@.springfield.io"},
            "type": "email_forwarding_change",
            "name": "email_forwarding_out_of_domain",
            "parameters": {"email_forwarding_destination_address": "HSimpson@example.com"},
        },
    ),
    RuleTest(
        name="Non Forwarding Event",
        expected_result=False,
        log={
            "id": {"applicationName": "user_accounts", "customerId": "D12345"},
            "actor": {"email": "homer.simpson@.springfield.io"},
            "type": "2sv_change",
            "name": "2sv_enroll",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteExternalMailForwarding(Rule):
    id = "GSuite.ExternalMailForwarding-prototype"
    display_name = "Gsuite Mail forwarded to external domain"
    enabled = False
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Collection:Email Collection", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0009:T1114"]}
    default_severity = Severity.HIGH
    default_description = "A user has configured mail forwarding to an external domain\n"
    default_reference = "https://support.google.com/mail/answer/10957?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Follow up with user to remove this forwarding rule if not allowed.\n"
    summary_attributes = ["p_any_emails"]
    tests = g_suite_external_mail_forwarding_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "user_accounts":
            return False
        if event.get("name") == "email_forwarding_out_of_domain":
            domain = deep_get(event, "parameters", "email_forwarding_destination_address").split("@")[-1]
            if domain not in config.GSUITE_TRUSTED_FORWARDING_DESTINATION_DOMAINS:
                return True
        return False

    def title(self, event):
        external_address = deep_get(event, "parameters", "email_forwarding_destination_address")
        user = deep_get(event, "actor", "email")
        return f"An email forwarding rule was created by {user} to {external_address}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcpk8s_pod_attached_to_node_host_network_tests: list[RuleTest] = [
    RuleTest(
        name="triggers",
        expected_result=True,
        log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                },
            ],
            "protoPayload": {"methodName": "io.k8s.core.v1.pods.create", "request": {"spec": {"hostNetwork": True}}},
        },
    ),
    RuleTest(
        name="ignore",
        expected_result=False,
        log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                },
            ],
            "protoPayload": {"methodName": "io.k8s.core.v1.pods.create", "request": {"spec": {"hostNetwork": False}}},
        },
    ),
]


@panther_managed
class GCPK8sPodAttachedToNodeHostNetwork(Rule):
    id = "GCP.K8s.Pod.Attached.To.Node.Host.Network-prototype"
    display_name = "GCP K8s Pod Attached To Node Host Network"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Optional"]
    default_severity = Severity.MEDIUM
    default_description = "This detection monitor for the creation of pods which are attached to the host's network. This allows a pod to listen to all network traffic for all deployed computer on that particular node and communicate with other compute on the network namespace. Attackers can use this to capture secrets passed in arguments or connections."
    reports = {"MITRE ATT&CK": ["TA0004:T1611"]}
    default_runbook = "Investigate a reason of creating a pod which is attached to the host's network. Advise that it is discouraged practice. Create ticket if appropriate."
    default_reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    tests = gcpk8s_pod_attached_to_node_host_network_tests

    def rule(self, event):
        if deep_get(event, "protoPayload", "methodName") not in (
            "io.k8s.core.v1.pods.create",
            "io.k8s.core.v1.pods.update",
            "io.k8s.core.v1.pods.patch",
        ):
            return False
        host_network = deep_walk(event, "protoPayload", "request", "spec", "hostNetwork")
        if host_network is not True:
            return False
        return True

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created or modified pod which is attached to the host's network in project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import okta_alert_context

okta_refresh_access_token_reuse_tests: list[RuleTest] = [
    RuleTest(
        name="Non-event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "123456",
                "displayName": "Okta User",
                "id": "okta.1234.",
                "type": "PublicClientApp",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Queens",
                    "country": "United States",
                    "geolocation": {"lat": 40, "lon": -70},
                    "postalCode": "11375",
                    "state": "New York",
                },
                "id": "okta.1234",
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "SAFARI",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "yyyy-abc-1111",
                    "behaviors": "{New Geo-Location=NEGATIVE, New Device=NEGATIVE, New IP=NEGATIVE, New State=NEGATIVE, New Country=NEGATIVE, Velocity=NEGATIVE, New City=NEGATIVE}",
                    "dtHash": "11aabbccc",
                    "grantType": "authorization_code",
                    "grantedScopes": "openid, profile, email, okta.users.read.self",
                    "redirectUri": "https://org.okta.com/enduser/callback",
                    "requestId": "ABCDEFG",
                    "requestUri": "/login/token/redirect",
                    "requestedScopes": "openid, profile, email, okta.users.read.self",
                    "responseMode": "query",
                    "responseType": "code",
                    "risk": "{level=LOW}",
                    "state": "SDFJDSLFS1234",
                    "threatSuspected": "false",
                    "url": "/login/token/redirect?stateToken=02.id.ASDDFJLKF",
                    "userId": "00abc124",
                },
            },
            "displaymessage": "OIDC authorization code request",
            "eventtype": "app.oauth2.authorize.code",
            "legacyeventtype": "app.oauth2.authorize.code_success",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-12-13 15:22:58.759",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Queens",
                            "country": "United States",
                            "geolocation": {"lat": 40, "lon": -70},
                            "postalCode": "11375",
                            "state": "New York",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {"id": "abcd123", "type": "User"},
                {"displayName": "Authorization Code", "id": "SDFSFJL", "type": "code"},
            ],
            "transaction": {"detail": {}, "id": "SDKFLSKLFJSLF", "type": "WEB"},
            "uuid": "abc-1234-aaa",
            "version": "0",
        },
    ),
    RuleTest(
        name="Reuse Event",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "123456",
                "displayName": "Okta User",
                "id": "okta.1234.",
                "type": "PublicClientApp",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Queens",
                    "country": "United States",
                    "geolocation": {"lat": 40, "lon": -70},
                    "postalCode": "11375",
                    "state": "New York",
                },
                "id": "okta.1234",
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "SAFARI",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "yyyy-abc-1111",
                    "behaviors": "{New Geo-Location=NEGATIVE, New Device=NEGATIVE, New IP=NEGATIVE, New State=NEGATIVE, New Country=NEGATIVE, Velocity=NEGATIVE, New City=NEGATIVE}",
                    "dtHash": "11aabbccc",
                    "grantType": "authorization_code",
                    "grantedScopes": "openid, profile, email, okta.users.read.self",
                    "redirectUri": "https://org.okta.com/enduser/callback",
                    "requestId": "ABCDEFG",
                    "requestUri": "/login/token/redirect",
                    "requestedScopes": "openid, profile, email, okta.users.read.self",
                    "responseMode": "query",
                    "responseType": "code",
                    "risk": "{level=LOW}",
                    "state": "SDFJDSLFS1234",
                    "threatSuspected": "false",
                    "url": "/login/token/redirect?stateToken=02.id.ASDDFJLKF",
                    "userId": "00abc124",
                },
            },
            "displaymessage": "Token Reuse",
            "eventtype": "app.oauth2.token.detect_reuse",
            "legacyeventtype": "app.oauth2.token.detect_reuse",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-12-13 15:22:58.759",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Queens",
                            "country": "United States",
                            "geolocation": {"lat": 40, "lon": -70},
                            "postalCode": "11375",
                            "state": "New York",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {"id": "abcd123", "type": "User"},
                {"displayName": "Authorization Code", "id": "SDFSFJL", "type": "code"},
            ],
            "transaction": {"detail": {}, "id": "SDKFLSKLFJSLF", "type": "WEB"},
            "uuid": "abc-1234-aaa",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaRefreshAccessTokenReuse(Rule):
    default_description = "When a client wants to renew an access token, it sends the refresh token with the access token request to the /token Okta endpoint.\nOkta validates the incoming refresh token, issues a new set of tokens and invalidates the refresh token that was passed with the initial request.\nThis detection alerts when a previously used refresh token is used again with the token request"
    default_reference = "https://developer.okta.com/docs/guides/refresh-tokens/main/#refresh-token-reuse-detection"
    display_name = "Okta App Refresh Access Token Reuse"
    default_runbook = "Determine if the clientip is anomalous. Revoke tokens if deemed suspicious."
    default_severity = Severity.MEDIUM
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.Refresh.Access.Token.Reuse-prototype"
    tests = okta_refresh_access_token_reuse_tests

    def rule(self, event):
        return event.get("eventtype") in ("app.oauth2.as.token.detect_reuse", "app.oauth2.token.detect_reuse")

    def title(self, event):
        return f"Okta Access Token Reuse Attempted by [{event.get('client', {}).get('ipAddress')}] [{event.get('actor', {}).get('displayName', '<no-displayname-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

github_repo_collaborator_change_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Collaborator Added",
        expected_result=True,
        log={
            "actor": "bob",
            "action": "repo.add_member",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "cat",
        },
    ),
    RuleTest(
        name="GitHub - Collaborator Removed",
        expected_result=True,
        log={
            "actor": "bob",
            "action": "repo.remove_member",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "cat",
        },
    ),
    RuleTest(
        name="GitHub - Non member action",
        expected_result=False,
        log={
            "actor": "bob",
            "action": "repo.enable",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "cat",
        },
    ),
]


@panther_managed
class GithubRepoCollaboratorChange(Rule):
    id = "Github.Repo.CollaboratorChange-prototype"
    display_name = "GitHub Repository Collaborator Change"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a repository collaborator is added or removed."
    default_runbook = "Determine if the new collaborator is authorized to access the repository."
    default_reference = "https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/managing-an-individuals-access-to-an-organization-repository"
    tests = github_repo_collaborator_change_tests

    def rule(self, event):
        return event.get("action") in ("repo.add_member", "repo.remove_member")

    def title(self, event):
        repo_link = f"https://github.com/{event.get('repo', '<UNKNOWN_REPO>')}/settings/access"
        action = "added to"
        if event.get("action") == "repo.remove_member":
            action = "removed from"
        return f"Repository collaborator [{event.get('user', '<UNKNOWN_USER>')}] {action} repository {event.get('repo', '<UNKNOWN_REPO>')}. View current collaborators here: {repo_link}"

    def severity(self, event):
        if event.get("action") == "repo.remove_member":
            return "INFO"
        return "MEDIUM"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_detection_alert_context, get_crowdstrike_field

crowdstrike_detectionpassthrough_tests: list[RuleTest] = [
    RuleTest(
        name="Low Severity Finding",
        expected_result=True,
        log={
            "cid": "11111111111111111111111111111111",
            "Technique": "PUP",
            "ProcessId": 377077835340488700,
            "AgentIdString": "00000000000000000000000000000000",
            "DetectName": "NGAV",
            "ComputerName": "macbook",
            "ProcessStartTime": "2021-09-18 20:38:51Z",
            "GrandparentCommandLine": "/sbin/launchd",
            "MACAddress": "aa-00-00-00-00-00",
            "CommandLine": "/Applications/app.app/Contents/MacOS/pup app",
            "Objective": "Falcon Detection Method",
            "Nonce": 1,
            "SHA256String": "3333333333333333333333333333333333333333333333333333333333333333",
            "ExternalApiType": "Event_DetectionSummaryEvent",
            "PatternDispositionValue": 2176,
            "DetectId": "ldt:00000000000000000000000000000000:222222222222222222",
            "Severity": 2,
            "PatternDispositionDescription": "Prevention/Quarantine, process was blocked from execution and quarantine was attempted.",
            "SeverityName": "Low",
            "MD5String": "33333333333333333333333333333333",
            "EventUUID": "33333333333333333333333333333333",
            "UserName": "bobert",
            "FilePath": "/Applications/app.app/Contents/MacOS/",
            "timestamp": "2021-09-18 20:38:52Z",
            "ParentCommandLine": "/usr/libexec/runningboardd",
            "DetectDescription": "This file is classified as Adware/PUP based on its SHA256 hash.",
            "LocalIP": "192.168.1.1",
            "ProcessEndTime": "1970-01-01 00:00:00Z",
            "SHA1String": "0000000000000000000000000000000000000000",
            "OriginSourceIpAddress": "",
            "GrandparentImageFileName": "/sbin/launchd",
            "MachineDomain": "",
            "ParentImageFileName": "/usr/libexec/runningboardd",
            "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity/detections/detail/00000000000000000000000000000000/222222222222222222?",
            "UTCTimestamp": "2021-09-18 20:38:52Z",
            "FileName": "pup app",
            "ParentProcessId": 376330001421757630,
            "EventType": "Event_ExternalApiEvent",
            "CustomerIdString": "11111111111111111111111111111111",
            "Tactic": "Malware",
            "SensorId": "00000000000000000000000000000000",
            "eid": 118,
            "PatternDispositionFlags": '{\n  "BlockingUnsupportedOrDisabled": false,\n  "BootupSafeguardEnabled": false,\n  "CriticalProcessDisabled": false,\n  "Detect": false,\n  "FsOperationBlocked": false,\n  "HandleOperationDowngraded": false,\n  "InddetMask": false,\n  "Indicator": false,\n  "KillActionFailed": false,\n  "KillParent": false,\n  "KillProcess": false,\n  "KillSubProcess": false,\n  "OperationBlocked": false,\n  "PolicyDisabled": false,\n  "ProcessBlocked": true,\n  "QuarantineFile": true,\n  "QuarantineMachine": false,\n  "RegistryOperationBlocked": false,\n  "Rooting": false,\n  "SensorOnly": false,\n  "SuspendParent": false,\n  "SuspendProcess": false\n}',
        },
    ),
    RuleTest(
        name="Low Severity Finding (FDREvent)",
        expected_result=True,
        log={
            "aid": "fa6a04a7f18d473fa06771b4961aa3d9",
            "cid": "712bcd164963442ea43d52917cecdecc",
            "ComputerName": "hostname.lan",
            "event": {
                "AgentIdString": "fa6a04a7f18d473fa06771b4961aa3d9",
                "CommandLine": "/bin/echo CROWDSTRIKE_SAMPLE_DETECTION",
                "ComputerName": "hostname.lan",
                "CustomerIdString": "712bcd164963442ea43d52917cecdecc",
                "DetectDescription": "Non-malicious sample detection generated for evaluation purposes.",
                "DetectId": "ldt:fa6a04a7f18d473fa06771b4961aa3d9:346037607921826886",
                "DetectName": "Suspicious Activity",
                "EventType": "Event_ExternalApiEvent",
                "EventUUID": "4624ba0c46c8405ea4998a0df7a5fa53",
                "ExternalApiType": "Event_DetectionSummaryEvent",
                "FalconHostLink": "https://falcon.us-2.crowdstrike.com/activity/detections/detail/00000000000000000000000000000000/222222222222222222?",
                "FileName": "echo",
                "FilePath": "/bin/",
                "GrandparentCommandLine": "-bash",
                "GrandparentImageFileName": "/bin/bash",
                "LocalIP": "192.168.86.255",
                "MACAddress": "a4-83-e7-19-28-8b",
                "MD5String": "2153a89b0a91c38f152acbefafd69b99",
                "MachineDomain": "",
                "Nonce": 1,
                "Objective": "N/A",
                "OriginSourceIpAddress": "",
                "ParentCommandLine": "bash",
                "ParentImageFileName": "/bin/bash",
                "ParentProcessId": 346017374056168100,
                "PatternDispositionDescription": "Detection, standard detection.",
                "PatternDispositionFlags": {
                    "BootupSafeguardEnabled": False,
                    "CriticalProcessDisabled": False,
                    "Detect": False,
                    "FsOperationBlocked": False,
                    "InddetMask": False,
                    "Indicator": False,
                    "KillParent": False,
                    "KillProcess": False,
                    "KillSubProcess": False,
                    "OperationBlocked": False,
                    "PolicyDisabled": False,
                    "ProcessBlocked": False,
                    "QuarantineFile": False,
                    "QuarantineMachine": False,
                    "RegistryOperationBlocked": False,
                    "Rooting": False,
                    "SensorOnly": False,
                },
                "PatternDispositionValue": 0,
                "ProcessEndTime": 1616609989,
                "ProcessId": 346037607908199600,
                "ProcessStartTime": 1616609989,
                "SHA1String": "0000000000000000000000000000000000000000",
                "SHA256String": "9e8a9e9f8cab6f07d8e711043bf856893660143aaf4385cb4e7b6faa97d6e61e",
                "SensorId": "fa6a04a7f18d473fa06771b4961aa3d9",
                "Severity": 2,
                "SeverityName": "Low",
                "Tactic": "N/A",
                "Technique": "N/A",
                "UTCTimestamp": 1616609989000,
                "UserName": "username",
                "cid": "712bcd164963442ea43d52917cecdecc",
                "eid": 118,
                "timestamp": "2021-03-24T18:19:49Z",
            },
            "fdr_event_type": "Event_DetectionSummaryEvent",
            "p_any_domain_names": ["hostname.lan"],
            "p_any_md5_hashes": [
                "2153a89b0a91c38f152acbefafd69b99",
                "712bcd164963442ea43d52917cecdecc",
                "fa6a04a7f18d473fa06771b4961aa3d9",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["9e8a9e9f8cab6f07d8e711043bf856893660143aaf4385cb4e7b6faa97d6e61e"],
            "p_any_trace_ids": ["712bcd164963442ea43d52917cecdecc", "fa6a04a7f18d473fa06771b4961aa3d9"],
            "p_any_usernames": ["russ"],
            "p_event_time": "2021-03-24 18:19:49",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-01-25 13:56:51.82",
            "p_row_id": "be33cdd7f1bfdba6f483df811601",
            "p_schema_version": 0,
            "timestamp": "2021-03-24 18:19:49",
        },
    ),
    RuleTest(
        name="Non-match (FDREvent)",
        expected_result=False,
        log={
            "cid": "11111111111111111111111111111111",
            "CommandLine": "/Applications/app.app/Contents/MacOS/pup app",
            "Objective": "Falcon Detection Method",
            "Nonce": 1,
            "SHA256String": "3333333333333333333333333333333333333333333333333333333333333333",
            "event": {"ExternalApiType": "something else"},
            "fdr_event_type": "something else",
            "PatternDispositionValue": 2176,
            "Severity": 2,
            "PatternDispositionDescription": "Prevention/Quarantine, process was blocked from execution and quarantine was attempted.",
            "SeverityName": "Low",
            "MD5String": "33333333333333333333333333333333",
            "eid": 118,
        },
    ),
]


@panther_managed
class CrowdstrikeDetectionpassthrough(Rule):
    id = "Crowdstrike.Detection.passthrough-prototype"
    display_name = "Crowdstrike Detection Passthrough"
    default_severity = Severity.MEDIUM
    log_types = [LogType.CROWDSTRIKE_DETECTION_SUMMARY, LogType.CROWDSTRIKE_FDR_EVENT]
    tags = ["Crowdstrike"]
    default_description = "Crowdstrike Falcon has detected malicious activity on a host."
    default_runbook = "Follow the Falcon console link and follow the IR process as needed."
    default_reference = "https://www.crowdstrike.com/blog/tech-center/hunt-threat-activity-falcon-endpoint-protection/"
    dedup_period_minutes = 0
    summary_attributes = ["p_any_ip_addresses"]
    tests = crowdstrike_detectionpassthrough_tests

    def rule(self, event):
        return get_crowdstrike_field(event, "ExternalApiType", default="none") == "Event_DetectionSummaryEvent"

    def title(self, event):
        return (
            f"Crowdstrike Alert ({get_crowdstrike_field(event, 'Technique')}) - "
            + f"{get_crowdstrike_field(event, 'ComputerName')}"
            + f"({get_crowdstrike_field(event, 'UserName')})"
        )

    def alert_context(self, event):
        return crowdstrike_detection_alert_context(event)

    def severity(self, event):
        return get_crowdstrike_field(event, "SeverityName")

    def dedup(self, event):
        return f"{get_crowdstrike_field(event, 'EventUUID')} " + f"- {get_crowdstrike_field(event, 'ComputerName')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_network_detection_alert_context

connectionto_embargoed_country_tests: list[RuleTest] = [
    RuleTest(
        name="Connection To CU",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "1431649125",
            "ContextProcessId": "1685738",
            "ContextTimeStamp": "2023-04-28 18:49:37.731",
            "Entitlements": "15",
            "InContext": "0",
            "aid": "877761efa8db44d7redacted",
            "aip": "1.1.1.1",
            "cid": "cfe6986909644340redacted",
            "event": {
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "1431649125",
                "ConnectionDirection": "0",
                "ConnectionFlags": "0",
                "ContextProcessId": "1685738",
                "ContextTimeStamp": "1682707777.731",
                "EffectiveTransmissionClass": "3",
                "Entitlements": "15",
                "EventOrigin": "1",
                "InContext": "0",
                "LocalAddressIP4": "10.0.0.1",
                "LocalPort": "137",
                "Protocol": "17",
                "RemoteAddressIP4": "152.206.0.1",
                "RemotePort": "443",
                "aid": "877761efa8db44d7redacted",
                "aip": "1.1.1.1",
                "cid": "cfe6986909644340redacted",
                "event_platform": "Win",
                "event_simpleName": "NetworkConnectIP4",
                "id": "34019b0c-c7de-4725-9f93-4b8d16688673",
                "name": "NetworkConnectIP4V12",
                "timestamp": "1682707778681",
            },
            "event_platform": "Win",
            "event_simpleName": "NetworkConnectIP4",
            "fdr_event_type": "NetworkConnectIP4",
            "id": "34019b0c-c7de-4725-9f93-4b8d16688673",
            "name": "NetworkConnectIP4V12",
            "p_any_ip_addresses": ["152.206.0.1", "10.0.0.1", "1.1.1.1"],
            "p_enrichment": {
                "greynoise_riot_advanced": {
                    "p_any_ip_addresses": [
                        {
                            "ip_cidr": "1.1.1.1/32",
                            "provider": {
                                "category": "public_dns",
                                "description": "Cloudflare, Inc. is an American...",
                                "explanation": "Public DNS services are used as...",
                                "name": "Cloudflare Public DNS",
                                "precedence": 0,
                                "trust_level": "1",
                            },
                            "scan_time": "2023-04-28 21:11:03.820349735",
                        },
                    ],
                },
                "ipinfo_asn": {
                    "p_any_ip_addresses": [
                        {
                            "asn": "AS27725",
                            "domain": "etecsa.cu",
                            "name": "Empresa de Telecomunicaciones de Cuba, S.A.",
                            "route": "152.206.0.0/17",
                            "type": "isp",
                        },
                        {
                            "asn": "AS13335",
                            "domain": "cloudflare.com",
                            "name": "Cloudflare, Inc.",
                            "route": "1.1.1.0/24",
                            "type": "hosting",
                        },
                    ],
                },
                "ipinfo_location": {
                    "p_any_ip_addresses": [
                        {
                            "city": "Matanzas",
                            "country": "CU",
                            "lat": "23.04111",
                            "lng": "-81.5775",
                            "postal_code": "",
                            "region": "Matanzas Province",
                            "region_code": "04",
                            "timezone": "America/Havana",
                        },
                        {
                            "city": "Los Angeles",
                            "country": "US",
                            "lat": "34.0522",
                            "lng": "-118.2437",
                            "postal_code": "90076",
                            "region": "California",
                            "region_code": "CA",
                            "timezone": "America/Los_Angeles",
                        },
                    ],
                },
                "ipinfo_privacy": {
                    "p_any_ip_addresses": [
                        {"hosting": True, "proxy": False, "relay": False, "service": "", "tor": False, "vpn": False},
                    ],
                },
            },
            "p_log_type": "Crowdstrike.FDREvent",
            "timestamp": "2023-04-28 18:49:38.681",
        },
    ),
    RuleTest(
        name="Google DNS",
        expected_result=False,
        log={
            "p_any_ip_addresses": ["8.8.8.8"],
            "p_enrichment": {
                "ipinfo_location": {
                    "p_any_ip_addresses": [
                        {
                            "city": "Mountain View",
                            "country": "US",
                            "lat": "37.4056",
                            "lng": "-122.0775",
                            "postal_code": "94043",
                            "region": "California",
                            "region_code": "CA",
                            "timezone": "America/Los_Angeles",
                        },
                    ],
                },
            },
            "p_log_type": "Crowdstrike.FDREvent",
        },
    ),
]


@panther_managed
class ConnectiontoEmbargoedCountry(Rule):
    default_description = "Detection to alert when internal asset is communicating with an sanctioned destination. This detection leverages Panther UDM and IPInfo enrichment."
    default_reference = "U.S. Sanctioned Destinations - https://www.bis.doc.gov/index.php/policy-guidance/country-guidance/sanctioned-destinations"
    display_name = "Connection to Embargoed Country"
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    id = "Connection.to.Embargoed.Country-prototype"
    default_severity = Severity.LOW
    tests = connectionto_embargoed_country_tests
    # U.S. Gov Sanctioned Destinations
    # Cuba
    # Iran
    # DPRK
    # Syria
    EMBARGO_COUNTRY_CODES = {"CU", "IR", "KP", "SY"}

    def get_enrichment_obj(self, event):
        return event.deep_get("p_enrichment", "ipinfo_location", "p_any_ip_addresses", default=None)

    def rule(self, event):
        enrichment_obj = self.get_enrichment_obj(event)
        # enrichment_object returns a list.
        # Iterate over list and check if the "country" value matches the country codes.
        if enrichment_obj:
            for i in enrichment_obj:
                if i.get("country") in self.EMBARGO_COUNTRY_CODES:
                    return True
        return False

    def title(self, event):
        enrichment_obj = self.get_enrichment_obj(event)
        country_codes = set(i.get("country") for i in enrichment_obj if i.get("country") in self.EMBARGO_COUNTRY_CODES)
        return f"Connection made to embargoed country: [{country_codes}]."

    def alert_context(self, event):
        if event.get("p_log_type") == "Crowdstrike.FDREvent":
            return crowdstrike_network_detection_alert_context(event) | {
                "p_any_ip_addresses": event.get("p_any_ip_addresses"),
            }
        return {"p_any_ip_addresses": event.get("p_any_ip_addresses")}
